var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

import { Component as ReactComponent, Children, createElement as h } from 'react';
import classnames from 'classnames';
import createEagerFactory from 'recompose/createEagerFactory';
import withState from 'recompose/withState';
import withReducer from 'recompose/withReducer';
import branch from 'recompose/branch';

var isFn = function isFn(test) {
  return typeof test === 'function';
};
var id = function id(Component) {
  return Component;
};
var getDisplayName = function getDisplayName(C) {
  return C.displayName || C.name || 'Component';
};

var Crate = function () {
  function Crate() {
    var hoc = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : id;

    _classCallCheck(this, Crate);

    this.hocFn = hoc;
    return this;
  }

  Crate.prototype.map = function map(fn) {
    return new Crate(fn(this.hocFn));
  };

  Crate.prototype.fold = function fold(fn) {
    return fn(this.hocFn);
  };

  Crate.prototype.compile = function compile(Component) {
    return this.fold(function (hoc) {
      return hoc(Component);
    });
  };

  Crate.prototype.hoc = function hoc(fn) {
    return this.map(function (hoc) {
      return function (Wrapped) {
        return hoc(fn(Wrapped));
      };
    });
  };

  // Inspect your props at a certain point


  Crate.prototype.inspect = function inspect() {
    return this.hoc(function (Wrapped) {
      var factory = createEagerFactory(Wrapped);
      return function (props) {
        console.log('inspect props: ' + getDisplayName(Wrapped), props);
        return factory(props);
      };
    });
  };

  // Set prop by key


  Crate.prototype.prop = function prop(key, value) {
    return this.hoc(function (Wrapped) {
      var factory = createEagerFactory(Wrapped);
      return function (props) {
        var _extends2;

        return factory(_extends({}, props, (_extends2 = {}, _extends2[key] = isFn(value) ? value(props) : value, _extends2)));
      };
    });
  };

  // Set className prop
  // accepts anything that `classnames`
  // https://www.npmjs.com/package/classnames


  Crate.prototype.className = function className(value) {
    return this.prop('className', function (props) {
      return classnames(props.className, isFn(value) ? value(props) : value);
    });
  };

  // Set style


  Crate.prototype.style = function style(s) {
    return this.prop('style', function (props) {
      return _extends({}, props.style, isFn(s) ? s(props) : s);
    });
  };

  Crate.prototype.lifecycle = function lifecycle(name, fn) {
    return this.hoc(function (Wrapped) {
      var factory = createEagerFactory(Wrapped);

      var Lifecycle = function (_ReactComponent) {
        _inherits(Lifecycle, _ReactComponent);

        function Lifecycle() {
          _classCallCheck(this, Lifecycle);

          return _possibleConstructorReturn(this, _ReactComponent.apply(this, arguments));
        }

        return Lifecycle;
      }(ReactComponent);

      Lifecycle.prototype.render = function () {
        return factory(this.props);
      };
      Lifecycle.prototype[name] = fn;
      return Lifecycle;
    });
  };

  return Crate;
}();

// Lifecycle


var lifecycleMethods = ['componentWillMount', 'componentDidMount', 'componentWillReceiveProps', 'shouldComponentUpdate', 'componentWillUpdate', 'componentDidUpdate', 'componentWillUnmount'];

lifecycleMethods.forEach(function (method) {
  Crate.prototype[method] = function (fn) {
    return this.lifecycle(method, fn);
  };
});

// Recompose utils

// https://github.com/acdlite/recompose/blob/master/docs/API.md#withstate
Crate.prototype.state = function state() {
  return this.hoc(withState.apply(undefined, arguments));
};

// https://github.com/acdlite/recompose/blob/master/docs/API.md#withreducer
Crate.prototype.reducer = function reducer() {
  return this.hoc(withReducer.apply(undefined, arguments));
};

// https://github.com/acdlite/recompose/blob/master/docs/API.md#branch
Crate.prototype.branch = function branch() {
  return this.hoc(branch.apply(undefined, arguments));
};

// factory
export default function crate() {
  for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  return new (Function.prototype.bind.apply(Crate, [null].concat(args)))();
}